// Datei ColumnNumberReader.java
/* ------------------------------------------------------------------------
A buffered character-input stream that keeps track of line numbers and
column numbers.

By default line- and column numbers will start at 0. This behavior
may be changed with the methods setLineNumber and setColumnNumber.
Example:
After the call setLineNumber(150) line   numbers will start at 150.
After the call setColumnNumber(5) column numbers will start at   5
(in each line read).

Terminology: In the documentation of the classes LineNumberReader and
ColumnNumberReader the term "character" denotes a single char value
(and never a character that is represented by two char values).

Most important object methods of this class:
read (a single character), readLine (read all characters,
getLineNumber, getColNumber,
setLineNumber, setColNumber.
---------------------------------------------------------------------------
Line terminators: In a textfile a line may be terminated in one of 3 ways:
- by a single newline         character '\n' (customary under Unix)   or
- by a single carriage return character '\r' (customary under Mac OS) or
- by the 2-character-sequence "\r\n"         (customary under Windows).
------------------------------------------------------------------------ */
import java.io.Reader;
import java.io.LineNumberReader;
import java.io.IOException;
import java.io.StringReader;
import java.io.FileNotFoundException;

public class ColumnNumberReader extends LineNumberReader {
   // ---------------------------------------------------------------------
   private int nextColumnNumber    = 0; // Changed by read and readLine
   private int numberOfFirstColumn = 0; // Changed by setColumnNumber only
   // ---------------------------------------------------------------------
   public ColumnNumberReader(Reader reader) {
      // Create a new line- and column number reader, using a buffer of
      // the default input-buffer size.
      super(reader);
   } // Constructor ColumnNumberReader
   // ---------------------------------------------------------------------
   public ColumnNumberReader(Reader reader, int bufferSize) {
      // Create a new line- and column number reader, using a buffer of
      // size bufferSize.
      super(reader, bufferSize);
   } // Constructor ColumnNumberReader
   // ---------------------------------------------------------------------
   @Override
   public String readLine() throws IOException {
      // Resets the nextColumnNumber to 0.
      // Then tries to read a line of text. If successful
      // increments the line number by 1 and returns the line read
      // as a string (without any line terminator at its end).
      // If reading a line is unsuccessful because there are no more
      // lines to be read, the value null will be returned.

      return "Error Error"; // MUST BE REPLACED
   } // readLine
   // ---------------------------------------------------------------------
   @Override
   public int read() throws IOException {
      // Reads and returns a single char value.
      // If a line terminator is read, then
      // - the line number      will be incremented by 1 and
      // - the nextColumnNumber will be reset       to 0 and
      // - a single newline character '\n' will be returned (even if the
      // line terminator read was a 2-charcter-sequence "\r\n").
      // If a char value other than a line terminator is read
      // the nextColumnNumber will be incremented by 1 and the
      // char value read will be returned.
      // If the end of the stream has been reached (no more characters
      // to be read) the value -1 will be returned.

      return -1; // MUST BE REPLACED
   } // read
   // ---------------------------------------------------------------------
   public int  getColumnNumber () {
      return -999; // MUST BE REPLACED
   }

   public void setColumnNumber (int numberOfFirstColumn) {
      ;           // MUST BE REPLACED
   } // setColNumber
   // ---------------------------------------------------------------------
   static public void main(String[] _) throws IOException {
      printf("---------------------------------------------------%n");
      printf("The class    ColumnNumberReader may be tested with %n" +
             "the program  ColumnNumberReaderIJut or with        %n" +
             "the program  ColumnNumberReaderITst                %n"
            );
      printf("---------------------------------------------------%n");

      // The following string contains 3 short lines of text:
      String             st = "ABC\nDE\r\nFGHI\r";
      StringReader       sr = new StringReader      (st);
      ColumnNumberReader cr = new ColumnNumberReader(sr);

      while (true) {
         int lnrBefore = cr.getLineNumber();
         int cnrBefore = cr.getColumnNumber();
         printf("(lnr: %2d, cnr: %2d)", lnrBefore, cnrBefore);
         int ci = cr.read();
         if (ci == -1) break;
         if (ci == '\n') {
            printf(" '\\n' ");
         } else {
            printf("  %c   ", (char) ci);
         }
         int lnrAfter = cr.getLineNumber();
         int cnrAfter = cr.getColumnNumber();
         printf("(lnr: %2d, cnr: %2d)%n", lnrAfter, cnrAfter);
      }
      printf("%n");
      printf("---------------------------------------------------%n");
   } // main
   // ---------------------------------------------------------------------
   // A method with a short name:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class ColumnNumberReader
/* ------------------------------------------------------------------------
Output of this programm should be:

---------------------------------------------------
The class    ColumnNumberReader may be tested with
the program  ColumnNumberReaderIJut or with
the program  ColumnNumberReaderITst
---------------------------------------------------
(lnr:  0, cnr:  0)  A   (lnr:  0, cnr:  1)
(lnr:  0, cnr:  1)  B   (lnr:  0, cnr:  2)
(lnr:  0, cnr:  2)  C   (lnr:  0, cnr:  3)
(lnr:  0, cnr:  3) '\n' (lnr:  1, cnr:  0)
(lnr:  1, cnr:  0)  D   (lnr:  1, cnr:  1)
(lnr:  1, cnr:  1)  E   (lnr:  1, cnr:  2)
(lnr:  1, cnr:  2) '\n' (lnr:  2, cnr:  0)
(lnr:  2, cnr:  0)  F   (lnr:  2, cnr:  1)
(lnr:  2, cnr:  1)  G   (lnr:  2, cnr:  2)
(lnr:  2, cnr:  2)  H   (lnr:  2, cnr:  3)
(lnr:  2, cnr:  3)  I   (lnr:  2, cnr:  4)
(lnr:  2, cnr:  4) '\n' (lnr:  3, cnr:  0)
(lnr:  3, cnr:  0)
---------------------------------------------------
------------------------------------------------------------------------ */
