// Datei ColumnNumberReaderJut.java
/* ------------------------------------------------------------------------
A JUnit testclass which tests the class ColumnNumberReader
(written with and for JUnit version 3.8, will not run with
later versions like 4.0 or 4.2).

The following methods (declared in ColumnNumberReader) are tested:
read, getLineNumber, getColumnNumber, setLineNumber, setColumnNumber and
readLine.
---------------------------------------------------------------------------
You can start this program in two ways:

1. By starting it like a normal program, e.g. with the command
> java ColumnNumberReaderJut

2. With the following command:
> java junit.awtui.TestRunner ColumnNumberReaderJut
---------------------------------------------------------------------------
Abbreviations:
sb  : should b
is  : is
sbd : should be data ("data as it should be") (German: Soll)
isd : is data        ("data as it is"       ) (German: Ist)
------------------------------------------------------------------------ */
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import java .io       .StringReader;
import java .io       .FileNotFoundException;
import java .io       .IOException;

public class ColumnNumberReaderJut extends TestCase {
   // ---------------------------------------------------------------------
   static public Test suite() {
      // The object returned contains (the names of) all procedures which
      // have 0 parameters and a name starting with "test".
      return new TestSuite(ColumnNumberReaderJut.class);
   } // suite
   // ---------------------------------------------------------------------
   // Some private auxiliary methods:
   // ---------------------------------------------------------------------
   private void doTheTesting1(
      String sbdChars,  // should be data Chars
      int[]  sbdLineNr, // should be data Line   numbers
      int[]  sbdColNr   // should be data Column numbers
   ){
      // Minimal parameters only:
      // For each char value in sbdChars there should be
      // a corresponding line number   in sbdLineNr and
      // a corresponding column number in sbdColNr.
      //
      // With the help of a StringReader object and a ColumnNumberReader
      // object each char value in sbdChars is read. The char value itself,
      // its line number and its column number are checked.

      checkLengths(sbdChars, sbdLineNr, sbdColNr, "doTheTesting1");

      StringReader       sr = new StringReader(sbdChars);
      ColumnNumberReader cr = new ColumnNumberReader(sr);

      doTheTesting3(cr, sbdChars, sbdLineNr, sbdColNr);
   } // doTheTesting
   // ---------------------------------------------------------------------
   private void doTheTesting2(
      String sbdChars,
      int[]  sbdLineNr,
      int[]  sbdColNr,
      int    lineNr,
      int    colNr
   ){
      // Minimal parameters plus a lineNr and a colNr:
      checkLengths(sbdChars, sbdLineNr, sbdColNr, "doTheTesting2");

      StringReader       sr = new StringReader(sbdChars);
      ColumnNumberReader cr = new ColumnNumberReader(sr);

      cr.setLineNumber (lineNr);
      cr.setColumnNumber(colNr);

      doTheTesting3(cr, sbdChars, sbdLineNr, sbdColNr);
   } // doTheTesting
   // ---------------------------------------------------------------------
   private void doTheTesting3(
      ColumnNumberReader cr,
      String             sbdChars,
      int[]              sbdLineNr,
      int[]              sbdColNr
   ){
      // Minimal parameters plus a ColumnNumberReader-object
      checkLengths(sbdChars, sbdLineNr, sbdColNr, "doTheTesting3");

      try {
         for (int i=0; i<sbdChars.length(); i++) {
            int sbLineNr   = sbdLineNr      [i];
            int sbColNr    = sbdColNr       [i];
            int sbChar     = sbdChars.charAt(i);

            int isLineNr   = cr.getLineNumber();
            int isColNr    = cr.getColumnNumber();
            int isChar     = cr.read();

            String msgLine = "Wrong line number:";
            String msgCol  = "Wrong column number:";
            String msgChar = makeMessage(sbChar, isChar);

            assertEquals(msgLine, sbLineNr, isLineNr);
            assertEquals(msgCol , sbColNr , isColNr );
            assertEquals(msgChar, sbChar  , isChar  );
         }
         // By now we should have reached the end of cr:
         assertEquals("Surplus char values at end:", -1, cr.read());
      } catch (IOException ex) {
         throw new Error("Error in doTheTesting3", ex);
      }
   } // doTheTesting
   // ---------------------------------------------------------------------
   private void checkLengths(String s, int[] a1, int[] a2, String name) {
      // Throws an exception if the length of s is not the same
      // as that of a1 and that of a2.
      if (s.length() == a1.length && s.length() == a2.length) return;
      String len1 = "" + s .length();
      String len2 = "" + a1.length;
      String len3 = "" + a2.length;
      String lengths = len1 + "/" + len2 + "/" + len3;
      throw new Error("Inconsisten lengths: " + lengths + " in " + name);
   } // checkLengths
   // ---------------------------------------------------------------------
   private String makeMessage(int sbChar, int isChar) {
      String sbS = (sbChar=='\n') ? "<\\n>" : "" + (char) sbChar;
      String isS = (isChar=='\n') ? "<\\n>" : "" + (char) isChar;
      return "expected:" + sbS + ", but was: " + isS + "! ";
   } // makeMessage
   // ---------------------------------------------------------------------
   // The public test cases:
   // Each test case consists mainly of
   // - a String-Object sbdChars ("should be data Chars"), which may
   //   contain one or several lines of data, separated with a single
   //   newline character '\n'
   // - an array sbdLineNr of line numbers, one for each char value
   //   in sbdChars
   // - an array sbdColNr of column numbers, one for each char value
   //   in sbdChars
   // ---------------------------------------------------------------------
   public void test_A() {
      // One line, no newline, line/col numbers default:
      String       sbdChars  = "ABC";
      int[]        sbdLineNr = {0, 0, 0};
      int[]        sbdColNr  = {0, 1, 2};

      doTheTesting1(sbdChars, sbdLineNr, sbdColNr);
   } // test_A
   // ---------------------------------------------------------------------
   public void test_B() {
      // One line, with newline, line/col numbers default:
      String       sbdChars  = "ABC\n";     // One line, ends with '\n'
      int[]        sbdLineNr = {0, 0, 0, 0};
      int[]        sbdColNr  = {0, 1, 2, 3};

      doTheTesting1(sbdChars, sbdLineNr, sbdColNr);
   } // test_B
   // ---------------------------------------------------------------------
   public void test_C() {
      // One line, no newline, line/col numbers 1/1:
      String       sbdChars  = "ABC";
      int[]        sbdLineNr = {1, 1, 1};
      int[]        sbdColNr  = {1, 2, 3};

      doTheTesting2(sbdChars, sbdLineNr, sbdColNr, 1, 1);
   } // test_C
   // ---------------------------------------------------------------------
   public void test_D() {
      // One line, with newline, line/col numbers 1/1:
      String       sbdChars  = "ABC\n";     // One line, ends with '\n'
      int[]        sbdLineNr = {1, 1, 1, 1};
      int[]        sbdColNr  = {1, 2, 3, 4};

      doTheTesting2(sbdChars, sbdLineNr, sbdColNr, 1, 1);
   } // test_D
   // ---------------------------------------------------------------------
   public void test_E() {
      // Severel lines, last line no newline, line/col numbers default:
      String       sbdChars  = "ABC\nDE\nFGHI";
      int[]        sbdLineNr = {0, 0, 0, 0, 1, 1, 1, 2, 2, 2, 2};
      int[]        sbdColNr  = {0, 1, 2, 3, 0, 1, 2, 0, 1, 2, 3};

      doTheTesting1(sbdChars, sbdLineNr, sbdColNr);
   } // test_E
   // ---------------------------------------------------------------------
   public void test_F() {
      // Severel lines, last line with newline, line/col numbers 5/3:
      String       sbdChars  = "ABC\nDE\nFGHI\n";
      int[]        sbdLineNr = {5, 5, 5, 5, 6, 6, 6, 7, 7, 7, 7, 7};
      int[]        sbdColNr  = {3, 4, 5, 6, 3, 4, 5, 3, 4, 5, 6, 7};

      doTheTesting2(sbdChars, sbdLineNr, sbdColNr, 5, 3);
   } // test_F
   // ---------------------------------------------------------------------
   public void test_readLine_A() {

      String             line1    = "ABC";
      String             line2    = "DE";
      String             line3    = "FGHI";
      String             sbdChars = line1+"\n"+line2+"\n"+line3+"\n";
      StringReader       sr       = new StringReader(sbdChars);
      ColumnNumberReader cr       = new ColumnNumberReader(sr);

      try {
         assertEquals( 0,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());

         assertEquals(line1, cr.readLine());

         assertEquals( 1,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());

         assertEquals(line2, cr.readLine());

         assertEquals( 2,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());

         assertEquals(line3, cr.readLine());

         assertEquals( 3,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());
      } catch (Exception ex) {
         fail("Unexpected exception: " + ex);
      }

   } // test_readLine_A
   // ---------------------------------------------------------------------
   public void test_readLine_B() {

      String             line1    = "ABC";
      String             line2    = "DE";
      String             line3    = "FGHI";
      String             sbdChars = line1+"\n"+line2+"\n"+line3;
      StringReader       sr       = new StringReader(sbdChars);
      ColumnNumberReader cr       = new ColumnNumberReader(sr);

      try {
         assertEquals( 0,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());

         assertEquals(line1, cr.readLine());

         assertEquals( 1,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());

         assertEquals(line2, cr.readLine());

         assertEquals( 2,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());

         assertEquals(line3, cr.readLine());

         assertEquals( 3,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());
      } catch (Exception ex) {
         fail("Unexpected exception: " + ex);
      }

   } // test_readLine_B
   // ---------------------------------------------------------------------
   public void test_readLine_C() {

      String             line1    = "";
      String             line2    = "DE";
      String             line3    = "";
      String             sbdChars = line1+"\n"+line2+"\n"+line3+"\n";
      StringReader       sr       = new StringReader(sbdChars);
      ColumnNumberReader cr       = new ColumnNumberReader(sr);

      try {
         assertEquals( 0,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());

         assertEquals(line1, cr.readLine());

         assertEquals( 1,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());

         assertEquals(line2, cr.readLine());

         assertEquals( 2,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());
         assertEquals(line3, cr.readLine());

         assertEquals( 3,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());
      } catch (Exception ex) {
         fail("Unexpected exception: " + ex);
      }

   } // test_readLine_C
   // ---------------------------------------------------------------------
   public void test_readLine_D() {
      // Both read and readLine are called here:

      String             sbdChars = "ABC\nDE\nFGHI\n";
      StringReader       sr       = new StringReader(sbdChars);
      ColumnNumberReader cr       = new ColumnNumberReader(sr);

      try {
         assertEquals( 0,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());

         assertEquals('A'  , cr.read    ());
         assertEquals( 0,    cr.getLineNumber());
         assertEquals( 1,    cr.getColumnNumber());
         assertEquals("BC" , cr.readLine());

         assertEquals( 1,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());

         assertEquals('D'  , cr.read    ());
         assertEquals( 1,    cr.getLineNumber());
         assertEquals( 1,    cr.getColumnNumber());
         assertEquals("E"  , cr.readLine());

         assertEquals( 2,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());

         assertEquals('F'  , cr.read    ());
         assertEquals( 2,    cr.getLineNumber());
         assertEquals( 1,    cr.getColumnNumber());
         assertEquals("GHI", cr.readLine());

         assertEquals( 3,    cr.getLineNumber());
         assertEquals( 0,    cr.getColumnNumber());
      } catch (Exception ex) {
         fail("Unexpected exception: " + ex);
      }

   } // test_readLine_D
   // ---------------------------------------------------------------------
   public void test_readLine_E() {
      // Both read and readLine are called here:

      String             sbdChars = "ABC\nDE\nFGHI\n";
      StringReader       sr       = new StringReader(sbdChars);
      ColumnNumberReader cr       = new ColumnNumberReader(sr);

      try {
         cr.setLineNumber  (3);
         cr.setColumnNumber(5);
         assertEquals( 3,    cr.getLineNumber());
         assertEquals( 5,    cr.getColumnNumber());

         assertEquals('A'  , cr.read    ());
         assertEquals( 3,    cr.getLineNumber());
         assertEquals( 6,    cr.getColumnNumber());
         assertEquals("BC" , cr.readLine());

         assertEquals( 4,    cr.getLineNumber());
         assertEquals( 5,    cr.getColumnNumber());

         assertEquals('D'  , cr.read    ());
         assertEquals( 4,    cr.getLineNumber());
         assertEquals( 6,    cr.getColumnNumber());
         assertEquals("E"  , cr.readLine());

         assertEquals( 5,    cr.getLineNumber());
         assertEquals( 5,    cr.getColumnNumber());

         assertEquals('F'  , cr.read    ());
         assertEquals( 5,    cr.getLineNumber());
         assertEquals( 6,    cr.getColumnNumber());
         assertEquals("GHI", cr.readLine());

         assertEquals( 6,    cr.getLineNumber());
         assertEquals( 5,    cr.getColumnNumber());
      } catch (Exception ex) {
         fail("Unexpected exception: " + ex);
      }

   } // test_readLine_E
   // ---------------------------------------------------------------------
   static public void main(String[] _) {
      // All test cases are executed:
      junit.awtui.TestRunner.run(ColumnNumberReaderJut.class);
   }
   // ---------------------------------------------------------------------
   // A convenience method with a short name:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class ColumnNumberReaderJut