// Datei KlammernPruefenJut.java
/* ------------------------------------------------------------------------
Diese Klasse dient zum Testen der Klasse KlammernPruefen.
---------------------------------------------------------------------------
Diese Klasse wurde mit dem Rahmenprogramm JUnit Version 3.8 erstellt.
Sie funktioniert nicht mit spaeteren Version von JUnit.

Man kann dieses Testprogramm auf zwei Weisen starten:

1. Indem man das Programm KlammernPruefenJut aufruft, etwa so:
> java KlammernPruefenJut

2. Indem man in einem bash- oder DOS-Fenster das folgende Kommando eingibt:
> java junit.awtui.TestRunner KlammernPruefenJut
------------------------------------------------------------------------ */
import junit.framework.Test;      // Eine Schnittstelle
import junit.framework.TestCase;  // Eine Test-Klasse
import junit.framework.TestSuite; // Eine Test-Klasse
import java.io.StringReader;
import java.io.IOException;

public class KlammernPruefenJut extends TestCase {
   // ---------------------------------------------------------------------
   static public Test suite() {
      // Alle parameterlosen Prozeduren dieser Klasse, deren Namen mit
      // "test" beginnen, werden zu einem TestSuite-Objekt zusammengefasst
      // und als Ergebnis geliefert:
      return new TestSuite(KlammernPruefenJut.class);
   } // suite
   // ---------------------------------------------------------------------
   // Private Hilfsmethoden
   // ---------------------------------------------------------------------
   private void do_AllesOk(String daten, String sollMsg) {

      KlammernPruefen.Ergebnis erg =
         doMost(daten, "KlammernPruefen$ErgAllesOk");

      // Jetzt wird der genaue Text des Ergebnisses erg geprueft:
      String  istMsg = erg.toString();
      assertEquals(sollMsg, istMsg);

   } // do_AllesOk
   // ---------------------------------------------------------------------
   private void do_FalscheKlammer(String daten, String sollMsg) {

      KlammernPruefen.Ergebnis erg =
         doMost(daten, "KlammernPruefen$ErgFalscheKlammer");

      // Jetzt wird der genaue Text des Ergebnisses erg geprueft:
      String istMsg = erg.toString();
      assertEquals(sollMsg, istMsg);

   } // do_FalscheKlammer
   // ---------------------------------------------------------------------
   private void do_KlammernFehlen(String daten, String sollMsg) {

      KlammernPruefen.Ergebnis erg =
         doMost(daten, "KlammernPruefen$ErgKlammernFehlen");

      // Jetzt wird der genaue Text des Ergebnisses erg geprueft:
      String  istMsg  = erg.toString();
      assertEquals(sollMsg, istMsg);

   } // do_KlammernFehlen
   // ---------------------------------------------------------------------
   private KlammernPruefen.Ergebnis doMost(String daten, String sollName) {
      StringReader       sr      = new StringReader(daten);
      ColumnNumberReader cr      = new ColumnNumberReader(sr);
      KlammernPruefen    kp      = new KlammernPruefen();

      KlammernPruefen.
      Ergebnis           erg     = kp.pruefeKlammern(cr);
      assertNotNull("pruefeKlammern hat null geliefert!", erg);

      String             istName = erg.getClass().getName();
      String msg = "Falscher Ergebnistyp beim Testen von \"" + daten + "\"";
      assertEquals(msg, sollName, istName);

      return erg;
   } // doMost
   // ---------------------------------------------------------------------
   // Die oeffentlichen Testfaelle
   // ---------------------------------------------------------------------
   public void test_AllesOk() {
      // Geprueft werden Zeichenfolgen, in denen alle Klammern ok sind

      // Die zu pruefenden Zeichenfolgen als Strings:
      String[] sr  = {
         "ABC",
         "ABC\nDE\nFGH",
         "A(B)[C]{D}E",
         "((A))[[B]]{{C}}",
         "([][])",
         "[{}{}]",
         "{()()}",
         "A\n([{\n}])B",
         "((([])[])[])",
      };

      // Das erwartete Ergebnis:
      String m = "Alle Klammern sind ok!";

      for (String daten : sr) do_AllesOk(daten, m);
   } // test_AllesOk
   // ---------------------------------------------------------------------
   public void test_FalscheKlammer() {
      // Geprueft werden Zeichenfolgen, in denen es zu einer schliessenden
      // Klammer keine zugehoerige oeffnende Klammer gibt.

      // Die zu pruefenden Zeichenfolgen als Strings:
      String[] sR  = {
         "AB(C]D",
         "AB{C{D}E)F",
         ")",
         "][",
         "}{}",
         "A\nB\nC[}D",
         "A\nB(\nC\nD}\n",

      };

      // Die erwarteten Ergebnisse:
      String[] mR = {
         "Klammer falsch: ], Zeile 1, Spalte 5",
         "Klammer falsch: ), Zeile 1, Spalte 9",
         "Klammer falsch: ), Zeile 1, Spalte 1",
         "Klammer falsch: ], Zeile 1, Spalte 1",
         "Klammer falsch: }, Zeile 1, Spalte 1",
         "Klammer falsch: }, Zeile 3, Spalte 3",
         "Klammer falsch: }, Zeile 4, Spalte 2",
      };

      checkLengths(sR, mR);

      for (int i=0; i<sR.length; i++) do_FalscheKlammer(sR[i], mR[i]);
   } // test_FalscheKlammer
   // ---------------------------------------------------------------------
   public void test_KlammernFehlen() {
      // Geprueft werden Zeichenfolgen, die nicht korrekt sind, aber
      // durch das Anhaengen von bestimmten schliessenden Klammern
      // korrekt gemacht werden koennten.

      // Die zu pruefenden Zeichenfolgen als Strings:
      String[] sR  = {
         "AB({}CD",
         "(([[{{",
         "A([]B{()C[()()D",
         "A(\n[{}{}]\n[[{",
      };

      // Die erwarteten Ergebnisse:
      String[] mR = {
         "Am Ende fehlen Klammern: )",
         "Am Ende fehlen Klammern: }}]]))",
         "Am Ende fehlen Klammern: ]})",
         "Am Ende fehlen Klammern: }]])",
      };

      checkLengths(sR, mR);

      for (int i=0; i<sR.length; i++) do_KlammernFehlen(sR[i], mR[i]);
   } // test_KlammernFehlen
   // ---------------------------------------------------------------------
   private void checkLengths(String[] sR, String[] mR) {
      // Wirft eine Ausnahme des Typs Error, wenn die Reihungen
      // sR und mR nicht gleich lang sind.

      if (sR.length == mR.length) return;
      String msg =
         "sR.length!=mR.length (" + sR.length + "!=" + mR.length +")";
      throw new Error(msg);
   } // checkLengths
   // ---------------------------------------------------------------------
   static public void main(String[] _) {
      // Alle Testfaelle werden ausgefuehrt:
      junit.awtui.TestRunner.run(KlammernPruefenJut.class);
   }
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class KlammernPruefenJut
