// Datei LineNumberReader02.java
/* ------------------------------------------------------------------
Dieses Programm zeigt:
Die Methode getLineNumber eines LineNumberReader-Objekts liefert
immer die Nr der Zeile, die als NAECHSTES gelesen wird (nicht die
Nr der ZULETZT GELESENEN Zeile).

Ein Zeilenwechsel in einer Textdatei wird (je nach Betriebssystem,
nach dessen Konventionen die Datei erstellt wurde) dargestellt
durch

- EIN Zeilenwechselzeichen '\n' (engl: newline, line feed)
  Unix, z.B. Linux
- EIN Wagenruecklaufzeichen '\r' (engl. carriage return)
  Mac OS
- ZWEI Zeichen "\r\n" (Wagenruecklauf und Zeilenwechsel)
  Windows

Der read-Befehl eines LineNumberReader-Objekts liefert einen
Zeilenwechsel immer in Form eines Zeilenwechselzeichens '\n'
(egal, aus wieviel und welchen Zeichen er tatsaechlich besteht).

Achtung: Die Reihenfolge von Zeichen ist wichtig:
"\r\n" (Wagenruecklauf, Zeilenwechsel) ist  EIN  Zeilenwechsel
       a la Windows.
"\n\r" (Zeilenwechsel, Wagenruecklauf) sind ZWEI Zeilenwechsel
       einer a la Unix und einer a la Mac OS
------------------------------------------------------------------ */
import java.io.Reader;
import java.io.StringReader;
import java.io.LineNumberReader;
import java.io.IOException;
import java.io.FileReader;

class LineNumberReader02 {
   // ---------------------------------------------------------------
   static public void main(String[] _) throws Exception {
      // Der Inhalt eines String-Objekts wird mit einem StringReader-
      // und eine LineNumberReader-Objekt zeichenweise gelesen.
      // Vor und nach dem Lesen eines Zeichens wird die Zeilen-Nr.
      // ermittelt (mit getLineNumber). Das Zeichen wird zwischen
      // diesen beiden Zeilen-Nrn. ausgegeben.

      String           str = "ABC\nDE\rFG\r\nHI\n\r";
      StringReader     sr  = new StringReader(str);
      LineNumberReader lnr = new LineNumberReader(sr);

      while (true) {
         // Vor dem Lesen eines Zeichens die Zeilen-Nr holen und ausgeben:
         int    lNrA  = lnr.getLineNumber();
         printf("line%2d ", lNrA);

         // Ein Zeichen als int-Wert lesen (koennte auch -1 sein):
         int    iChar = lnr.read();

         // Wenn der Text fertig gelesen ist:
         if (iChar == -1) break;

         // Den int-Wert iChar in einen lesbaren String umwandeln
         // (d.h. die Zeichen '\n' und '\r' lesbar machen):
         String sChar;
         switch (iChar) {
            case '\n': sChar = "'\\n'"; break;
            case '\r': sChar = "'\\r'"; break;
            default:   sChar = "'" + (char) iChar + "'";
         }

         // Nach dem Lesen eines Zeichens die Zeilen-Nr holen und
         // (zusammen mit dem Zeichen) ausgeben:
         int    lNrB = lnr.getLineNumber();
         printf("%4s line%2d%n", sChar, lNrB);
      } // while

   } // main
   // ---------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {
      System.out.printf(f, v);
   } // printf
   // ---------------------------------------------------------------
} // class LineNumberReader02
/* ------------------------------------------------------------------------
Ausgabe:                // Von Hand hinzugefuegte
                        // Kommentare:
line 0  'A' line 0
line 0  'B' line 0
line 0  'C' line 0
line 0 '\n' line 1      // Wechsel zwischen Zeile 0 und 1
line 1  'D' line 1
line 1  'E' line 1
line 1 '\n' line 2      // Wechsel zwischen Zeile 1 und 2
line 2  'F' line 2
line 2  'G' line 2
line 2 '\n' line 3      // Wechsel zwischen Zeile 2 und 3
line 3  'H' line 3
line 3  'I' line 3
line 3 '\n' line 4      // Wechsel zwischen Zeile 3 und 4, Zeile 4 ist leer
line 4 '\n' line 5      // Wechsel zwischen Zeile 4 und 5, Zeile 5 ist leer
line 5
------------------------------------------------------------------------ */
