// Datei MethodTest.java
/* ------------------------------------------------------------------------
This class should make it easier, to test methods. It supports the
testing of both class- and object-methods (static and non-static methods).
It supports the testing of functions (value returning methods).

Procedures (void-functions in C-speak) may be tested too, but only if
they output "their result" with methods named p und pln. Normally, "p"
and "pln" are abbreviations for "System.out.print" and "System.out.println"
respectively, and these methods print data to the standard output (the
screen, if not redirected). To test a procedure, special implementations
of these methods should be used, which "print" data to a StringBuilder
object named iScreen. Such implementations of p and pln are declared as
class methods (static methods) of this class.

This class allows to specify many tests with a minimum of lines of code.
Examples and detailed instructions on how to use it may be found in the
files MethodTestTstA.java and MethodTestTstB.java.

Since this class MethodTest belongs to a named package, its static members
(the methods p, pln and totalsString)  can be imported statically and may
then be denoted by their simple names (for an example see the file
MethodTestTstA.java).
------------------------------------------------------------------------ */
package de.bht_berlin.fb6;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.List;
import java.util.Arrays;
import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;

//import java.nio.HeapCharBuffer;

public class MethodTest {
   // ---------------------------------------------------------------------
   // Conveniance constructor with only one param:
   public MethodTest(Object[][] aa) {
      this(aa, true);
   } // constructor MethodTest
   // ---------------------------------------------------------------------
   // The constructor that does most of the work:
   public MethodTest(Object[][] aa, boolean failuresOnly) {
      this.aa           = aa;
      this.failuresOnly = failuresOnly;

      checkPlausibilityA(aa);
      initMethodOb      (aa);
      checkPlausibilityB(aa, methodOb);
      initCallA         (aa);
      execute(callA);

      printf("--------------------------------------%n");
      printf("%s", this.toString());

   } // constructor MethodTest
   // ---------------------------------------------------------------------
   static public MethodTest test(Object[][] aa) {
      // Convenience method, may be simpler to call than a constructor:
      return new MethodTest(aa, true);
   }

   static public MethodTest test(Object[][] aa, boolean failuresOnly) {
      // Convenience method, may be simpler to call than a constructor:
      return new MethodTest(aa, failuresOnly);
   }
   // ---------------------------------------------------------------------
   public void setObject(Object objectOb) {
      aa[0][0]          = objectOb;
      this.nrOfCalls    = 0;
      this.nrOfFails    = 0;

      checkPlausibilityA(aa);
      initMethodOb      (aa);
      checkPlausibilityB(aa, methodOb);
      initCallA         (aa);
      execute(callA);

      printf("--------------------------------------%n");
      printf("%s", this.toString());

   } // setMethod
   // ---------------------------------------------------------------------
   public boolean getFailuresOnly() {return failuresOnly;}
   public boolean setFailuresOnly(boolean failuresOnly) {
      boolean res = this.failuresOnly;
      this.failuresOnly = failuresOnly;
      return res;
   } // setFailuresOnly
   // =====================================================================
   // The object iScreen (together with the public class methods named
   // p and pln) realises a kind of internal screen, which is useful to
   // test pocedures which print data to a screen.

   static private StringBuilder iScreen = new StringBuilder();

   static public void p  (Object ob){iScreen.append(ob.toString())       ;}
   static public void pln(Object ob){iScreen.append(ob.toString()); pln();}
   static public void pln()         {iScreen.append("\n")                ;}
   // -------------------------------------------
   static private String getAndClearIScreenContent() {
      String res = iScreen.toString();     // get the content
      iScreen.delete(0, iScreen.length()); // clear the iScreen
      return res;                          // return the content as result
   } // getAndClearIScreenContent
   // =====================================================================
   // The fields of a MethodTest object:
   private Object[][] aa;        // The array of arrays
   private Class<?>   classOb;   // The class declaring methodOb
   private Method     methodOb;  // The method to be tested
   private Object     objectOb;  // For object methods
   private Class<?>[] parTypeA;  // params types array for methodOb
   private Call    [] callA;     // call array

   private boolean
      failuresOnly,  // print failures only
      isFunction,    // methodOb is a function (no procedure)
      isStatic;      // methodOb is a class method
   private int
      nrOfCalls,     // Number of calls (ok ones and failures)
      nrOfFails;     // Number of failures
   private int[]
      colW;          // Column widths, for a columnar output format
   static private int
      totalCalls,    // Number of calls    alltogether
      totalFails;    // Number of failures alltogether
   // =====================================================================
   private class Call {
      // Each object of this inner class represents one call of the method
      // to be tested, including the expected and the as-is result.

      Object     resXpOb;    // Result as expected
      Object     resIsOb;    // Result as is
      Object[]   parValA;    // Params for methodOb

      boolean    isOk;       // The execution produced no failure
      // ------------------------------------------------------------------
      public String toString() {
         StringBuilder sb = new StringBuilder();

         // To arrange data in a columnar format, the maximum lengths of
         // corresponding items in each line is calculated before the
         // formating starts in ernest:
         if (colW == null) colW = colWidths(callA, failuresOnly);


         int nrp = callA[0].parValA.length; // Nr of parameters
         int iXp = nrp + 0;                 // Index of expected result
         int iIs = nrp + 1;                 // Index of       is-result

         if (isStatic) {
            sb.append(classOb.getSimpleName());
         } else {
            sb.append(tos(objectOb));
         }
         sb.append("."); // Dot notation

         // Include the method call as a String:
         sb.append(methodOb.getName() + "(");
         if (parValA.length > 0) {
            sb.append(tos(parValA[0], colW[0]));
            for (int i=1; i<parValA.length; i++) {
               sb.append(",");
               sb.append(tos(parValA[i], colW[i]));
            }
         }

         // Include the expected result and the is-result:
         sb.append("): xp:");
         sb.append(tos(resXpOb, colW[iXp]));
         sb.append(", is:");
         sb.append(tos(resIsOb, colW[iIs]));

         // Make failures hard to overlook:
         if (isOk) {
            sb.append(" o.k.");
         } else {
            sb.append(" ****");
         }

         return sb.toString();
      } // toString
      // ------------------------------------------------------------------
   } // class Call
   // =====================================================================
   private void checkPlausibilityA(Object[][] aa) {
      // Checks several traits of aa and throws an Error object if
      // something is not as expected.

      // There should be at least the special component aa[0] and
      // one description of a test call in    component aa[1]:
      if (aa.length < 2) {
         err("%nIn method checkAA:%n" +
         "Length of array aa should be greater than 1%n" +
         "but is %d !", aa.length);
      }

      // In aa[0]
      // There should be at least a (Class-) object aa[0][0] and
      // the name of the method to be tested        aa[0][1]:
      if (aa[0].length < 2) {
         err("%nIn method checkPlausibility:%n" +
         "Length of aa[0] should greater than 1, but is %d!",
         aa[0].length);
      }

      Class<?> c01 = aa[0][1].getClass();

      // The name of the method to be tested aa[0][1] should be a String:
      if (c01 != String.class) {
         err("%nIn method checkPlausibility:%n" +
         "Type of aa[0][1] should be String, but is %s!",
         c01.getName());
      }

      // All components of aa[0] after the first two should be of type
      // Class (describing the param types of the method to be tested):
      for (int i=2; i<aa[0].length; i++) {
         Class c0i = aa[0][i].getClass();
         if (c0i != Class.class) {
            err("%nIn method checkPlausibility:%n" +
            "Type of aa[0][%d] should be Class, but is %s!",
            i, c0i.getName());
         }
      }

      // The number of param types in aa[0] (if any are present) should
      // agree with the number of param values in aa[1]:
      if (aa[0].length > 2 && aa[0].length != aa[1].length+1) {
         err("%nIn method checkPlausibility:%n" +
         "aa[0].length - 2: %d and%n"              +
         "aa[1].lengtn - 1: %d should be equal but are not!",
         aa[0].length, aa[1].length+1);
      }

      // All components of aa except aa[0] should have equal lengths:
      for (int i=2; i<aa.length; i++) {
         int len1 = aa[1].length;
         int leni = aa[i].length;
         if (len1 != leni) {
            err("%nIn method checkPlausibility:%n" +
            "Length of aa[%d] is: %d, but%n"       +
            "Length of aa[%d] is: %d!", 1, len1, i, leni);
         }
      }

      // If no param types are specified in aa[0] we are finished:
      if (aa[0].length==2) return;

      // If param types are specified in aa[0], are all the param values
      // in all the calls of the correct types? More precisely:
      // For each call  number cnr in [1 ..    aa.length)    and
      // for each param number pnr in [1 .. aa[1].length)
      // the param value in aa[cnr][pnr] should be of type aa[0][pnr+1]
      for (int cnr=1; cnr<aa.length; cnr++) {
         for (int pnr=1; pnr<aa[1].length; pnr++) {
            Object parVal  =         aa[cnr][pnr  ];
            Class  parType = (Class) aa[ 0 ][pnr+1];
            if (! isCompatible(parVal, parType)) {
               err("%nIn method checkPlausibility:%n"          +
               "The param value aa[%d][%d]: %s should be of%n" +
               "the param type  aa[%d][%d]: %s but is of   %n" +
               "the       type          : %s!",
               cnr, pnr  , tos(parVal),
               0  , pnr+1, parType.getName(),
               parVal.getClass().getName());
            }
         } // for pnr
      } // for cnr

   } // checkPlausibilityA
   // ---------------------------------------------------------------------
   private void checkPlausibilityB(Object[][] aa, Method methodOb) {
      try {
         // A final check: Are the expected results in aa
         // compatible with the result type of the methodOb?

         // Get the result type of the method to be testet:
         // For a function,  the result type is its return type.
         // For a procedure, the result type is String (the content
         // of the internal screen iScreen):
         Class<?> resultType = methodOb.getReturnType();
         if (!isFunction) resultType = String.class;

         for (int cnr=1; cnr<aa.length; cnr++) {
            Object returnVal = aa[cnr][0];
            if (!isCompatible(returnVal, resultType)) {
               err("%nIn method checkPlausibilityB:%n"            +
               "The return value aa[%2d][%2d]: %s should be of%n" +
               "the return type            : %s               %n" +
               "of  the method             : %s, but is of    %n" +
               "the type                   : %s!",
               cnr, 0  , tos(returnVal), resultType.getName(),
               methodOb.getName(), returnVal.getClass().getName());
            }
         }
      } catch (Exception ex) {
         throw new Error("In method checkPlausibilityB:", ex);
      }
   } // checkPlausibilityB
   // ---------------------------------------------------------------------
   static public boolean isCompatible(Object ob, Class<?> cl) {
      // Returns true
      // if ob has the value null   or
      // if ob is an instance of cl or
      // if ob is an instance of a wrapper class (e.g. Integer) and
      // cl represents the corresponding primitive type (i.e. Integer.TYPE)

      if (ob == null)            return true;
      if (cl.isInstance(ob))     return true;
      int indWrapperClass  = wpTypeList.indexOf(ob.getClass());
      if (indWrapperClass == -1) return false;
      int indPrimitiveType = wpTypeList.indexOf(cl);
      return indWrapperClass+1 == indPrimitiveType;
   } // isCompatible
   // ---------------------------------------------------------------------
   private void initMethodOb(Object[][] aa) {

      String methodName = (String) aa[0][1];

      try {
         if (aa[0].length > 2) {
            // If param types are present in aa[0] they will be used to
            // identify and get the Method object to be tested:
            int len = aa[0].length-2;
            this.parTypeA = new Class<?>[len];
            for (int i=0; i<len; i++) parTypeA[i] = (Class<?>) aa[0][i+2];
         } else {
            // Get the types of the parameters of the call described in aa[1]:
            int len = aa[1].length-1;
            this.parTypeA = new Class<?>[len];
            for (int i=0; i<len; i++) parTypeA[i] = aa[1][i+1].getClass();

            // Primitive params will be represented by wrapper objects.
            // Getting the class of such an object, we get the wrapper
            // class instead of the correct primitive type.
            // Strategy: By default, we consider wrapper obects to represent
            // primitive values. If the method to be tested has wrapper
            // params, the user has to specify their types in aa[0]
            makePrimitive(parTypeA);
         }

         // Get the Class object which contains the method to be tested:
         this.objectOb =  aa[0][0];
         if (objectOb instanceof Class<?>) {
            classOb  = (Class<?>) objectOb;
         } else {
            classOb = objectOb.getClass();
         }

         // Try to get the Method object of the method to be tested:
         this.methodOb   = classOb.getDeclaredMethod(methodName, parTypeA);
         this.isFunction = (this.methodOb.getReturnType() != void.class);
         this.isStatic   = (methodOb.getModifiers()&Modifier.STATIC) != 0;

      } catch (NoSuchMethodException noSuch1) {            // catch-block-1
         // Are we in the rare situation that an object method declared
         // in class Class is to be tested?
         if (! (objectOb instanceof Class<?>)) {
            err("%nIn method initMethodOb, catch block 1:%n" +
            "There is no method %s in class %s with param types%n%s !",
            methodName, classOb.getName(), Arrays.toString(parTypeA));
         }

         try {
            // The objectOb is a Class object.
            // Try to get the method object to be tested from class Class:
            this.methodOb   = Class.class.getMethod(methodName, this.parTypeA);
            this.isFunction = (this.methodOb.getReturnType() != void.class);
            this.isStatic   = (methodOb.getModifiers()&Modifier.STATIC) != 0;
         } catch (NoSuchMethodException noSuch2) {
            err("%nIn method initMethod, catch in catch:"  +
            "%nCould not find a declaration of method %s!" +
            "%nCause 1: %s"                                +
            "%nCause 2: %s",
            methodName, noSuch1, noSuch2);
         }
      } catch (Exception ex) {                             // catch-block-2
         throw new Error("In method initMethodOb:", ex);
      }
   } // initMethodOb
   // ---------------------------------------------------------------------
   // Problem: If c1 and c2 are Class objects representing a primitive type
   // and its wrapper class (e.g. Integer.TYPE and Integer.class) then the
   // following expressions both have the value false:
   //
   // c1.isAssignableFrom(c2)
   // c2.isAssignableFrom(c1)
   //
   // In a sense, this is "not true anymore": Since auto boxing was intro-
   // duced with Java 5.0 in the year 2004 both expressions should have
   // the value true.
   //
   // The collection wpTypeList (w like wrapper, p like primitive) defined
   // below helps to define "the true isAssignableFrom relation"  and to
   // solve some related problems (see the methods makePrimitive, tos and
   // isCompatible).

   static final Class[] wpTypeArray = {
      Byte     .class, Byte     .TYPE,
      Character.class, Character.TYPE,
      Short    .class, Short    .TYPE,
      Integer  .class, Integer  .TYPE,
      Long     .class, Long     .TYPE,
      Float    .class, Float    .TYPE,
      Double   .class, Double   .TYPE,
      Boolean  .class, Boolean  .TYPE,
   };

   static final List<Class> wpTypeList = Arrays.asList(wpTypeArray);

   private void makePrimitive(Class<?>[] cr) {
      // Every component of cr which represents a wrapper class is
      // changed such that it represents the corresponding primitive type.

      for (int i=0; i<cr.length; i++) {
         int j = wpTypeList.indexOf(cr[i]);
         if (j%2==0) cr[i] = wpTypeArray[j+1];
      }
   } // makePrimitive
   // ---------------------------------------------------------------------
   private void initCallA(Object[][] aa) {
      // Initializes the field callA with data taken from aa. Each
      // component of the array callA represents one call of the
      // method to be tested.

      callA = new Call[aa.length-1];

      for (int i=0; i<callA.length; i++) {
         Object[] r = aa[i+1];
         Call     c = callA[i] = new Call(); // Tricky double assignment!

         c.resXpOb  = r[0];
         c.parValA  = Arrays.copyOfRange(r, 1, r.length);
      }

      nrOfCalls = callA.length;
   } // initCallA
   // ---------------------------------------------------------------------
   private void execute(Call[] ca) {
      // Executes each method call in ca.

      for (Call c : ca) {
         try {
            // The method to be testet is called:
            c.resIsOb  = methodOb.invoke(objectOb, c.parValA);

            // If the method is a procedure, its result has to be fetched
            // from the internal screen iScreen:
            if (!isFunction) c.resIsOb = getAndClearIScreenContent();

            // The call is o.k. (is no failure) if no exception was thrown
            // and the expected result agrees with the is-result:

            if (c.resXpOb == null) {
               c.isOk = c.resIsOb == null;
            } else {
               c.isOk = deepEquals(c.resXpOb, c.resIsOb);
            }

            if (!c.isOk) nrOfFails++;
         } catch (InvocationTargetException ite) {
            c.resIsOb = ite.getCause().getMessage();
            nrOfFails++;
         } catch (IllegalAccessException iae) {
            c.resIsOb = iae.getMessage();
            nrOfFails++;
         }
      }

      // totalCalls and totalFails are class fields:
      totalCalls += this.nrOfCalls;
      totalFails += this.nrOfFails;
   } // execute
   // ---------------------------------------------------------------------
   private static boolean deepEquals(Object ob1, Object ob2) {
      // Returns true if and only if ob1 and ob2 are equal and a
      // certain generally useful sense :-)
      // Functions as expected if ob1 and ob2 are objects of
      // - type StringBuilder
      // - an array type with primitive or non-primitive components

      // Return the appropriate value if at least one parameter is null:
      if (ob1 == null) return ob2 == null;
      if (ob2 == null) return false;

      // Now both ob1 and ob2 are not null.
      Class<?> c1 = ob1.getClass();
      Class<?> c2 = ob2.getClass();

      // To be equal, ob1 and ob2 have to have exactly the  same type:
      if (c1 != c2) return false;

      // Objects of type StringBuilder are compared after conversion
      // to String:
      if (c1 == StringBuilder.class) {
         return ob1.toString().equals(ob2.toString());
      }

      // Obects which are not arrays are compared simply with the
      // equals method of ob1:
      if (!c1.isArray()) return ob1.equals(ob2);

      // Now ob1 and ob2 are arrays of the same type

      // To be equal the arrays have to be of the same length:
      int len1 = Array.getLength(ob1);
      int len2 = Array.getLength(ob2);

      if (len1 != len2) return false;

      // The arrays ob1 and ob2 are copied into variables of the
      // array type Object[] ...
      Object[] obA1 = new Object[len1];
      Object[] obA2 = new Object[len1];

      for (int i=0; i<len1; i++) {
         obA1[i] = Array.get(ob1, i);
         obA2[i] = Array.get(ob2, i);
      }

      // ... and compared with deepEquals of class Arrays:
      return Arrays.deepEquals(obA1, obA2);
   } // deepEquals
   // ---------------------------------------------------------------------
   public String toString() {
      // Returns a string representation of this MethodTest object,
      // containing all calls or only failures depending on failuresOnly.
      //
      // Note: Here each line end has to be represented by a single
      // newline character '\n' (and not by '%n' the meaning of which
      // depends on the platform used), otherwise the removel of
      // surplus empty lines (performed at the end of this method)
      // will not work reliably.

      StringBuilder sb = new StringBuilder();

      // Class- and method-name are always included:
      sb.append(String.format("Class : %s, ", classOb.getName() ));
      sb.append(String.format("Method: %s\n", methodOb.getName()));
      // The object is only included for object methods:
      if (!isStatic)
      sb.append(String.format("Object: %s\n", tos(objectOb)     ));

      // Include all calls or only all failures:
      for (Call c : callA) {
         if (failuresOnly && c.isOk) continue;
         sb.append("\n");
         sb.append(c.toString());
      }

      // Include a short summary:
      sb.append(String.format("\n\nNumber of calls: %2d, failures: %2d\n",
         nrOfCalls, nrOfFails));

      // Replace each sequence of two empty lines by one empty line:
      int n;
      while ((n=sb.indexOf("\n\n\n"))!=-1) sb.replace(n, n+3, "\n\n");

      return sb.toString();
   } // toString
   // ---------------------------------------------------------------------
   public int[] colWidths(Call[] ca, boolean failuresOnly) {
      // To arrange the content of this object in a columnar format,
      // the method toString needs to know the maximum lengths of all the
      // data items which are to appear in the same column. Format of
      // the array ret, which is returned by this method:
      // ret[0]   Max length of first    params
      // ret[1]   Max length of second   params
      // ...
      // ret[iXp] Max length of expected results
      // ret[iIs] Max length of       is-results

      int nrp = ca[0].parValA.length; // Nr of parameters
      int iXp = nrp + 0;
      int iIs = nrp + 1;

      int[] ret = new int[nrp+2];    // params plus resXp and resIs

      for (Call c : ca) {
         if (failuresOnly && c.isOk) continue;
         int lenXp = tos(c.resXpOb).length();
         int lenIs = tos(c.resIsOb).length();

         if (ret[iXp] < lenXp) ret[iXp] = lenXp;
         if (ret[iIs] < lenIs) ret[iIs] = lenIs;

         Object[] pA = c.parValA;
         for (int i=0; i<pA.length; i++) {
            int len = tos(pA[i]).length();
            if (ret[i] < len) ret[i] = len;
         }
      }

      return ret;
   } // colWidths
   // ---------------------------------------------------------------------
/*
   public String tos(Object ob) { // A to-string-method with a short name
      // Returns       null as "null",
      // String        objects enclosed in double quotes:    "..."
      // StringBuilder objects enclosed in double quotes:    "..."
      // Character     objects enclosed in single quotes:    '...'
      // Wrapper       objects without enclosing characters:  ...
      // other         objects enclosed in brackets:         [...]

      if (ob == null)                  return "null";
      String s = repCrAndNl(ob.toString());
      if (ob instanceof String)        return '"' + s + '"';
      if (ob instanceof StringBuilder) return '"' + s + '"';
      if (ob instanceof Character)     return "'" + s + "'";

      // Is ob an object of a wrapper class?
      if (wpTypeList.contains(ob.getClass())) return s;

      return "[" + s + "]";
   } // tos
*/
   // ---------------------------------------------------------------------
   public String tos(Object ob) {// A to-string-method with a short name
      // Tries to return a useful String representation of ob.
      // If ob is of type String or Character, a quoted string
      // (e.g. "ABC" or 'A') is returned.
      // If ob is an array, a string of the form "[...]" is returned,
      // where ... stands for String representations of the components,
      // separated (as usual) by ", ".
      // If ob is null, "null" is returned. Examples:
      //                                         conten:         length
      // tos(new StringBuilder("ABC"))  is  ABC               3
      // tos("ABC")                     is "ABC"              5
      // tos('A')                       is 'A'                3
      // tos(new Character('A'))        is 'A'                3
      // tos(new char[]{'A', 'B', 'C'}) is ['A', 'B', 'C']   15
      // tos(null)                      is null               4

      if (ob==null) return "null";

      final String SQ = "'";   // single quote
      final String DQ = "\"";  // double quotes

      Class<?> clOb = ob.getClass();

      if (clOb==String   .class) return DQ + ob + DQ;  // double quoted
      if (clOb==Character.class) return SQ + ob + SQ;  // single quoted
      if (! clOb.isArray())      return ob.toString(); // The usual String

      // Now ob is a runtime array! We compute its String representation
      // component by component:

      int           obLen = Array.getLength(ob);
      StringBuilder sb    = new StringBuilder();
      sb.append("[");
      if (obLen>0) {
         sb.append(tos(Array.get(ob, 0)));
         for (int i=1; i<obLen; i++) {
            sb.append(", " + tos(Array.get(ob, i)));
         }
      }
      sb.append("]");
      return sb.toString();
   } // tos
   // ---------------------------------------------------------------------
   public String tos(Object ob, int width) { // Another to-string-method
      // Combines tos-with-1-param with formatting to a certain width.
      // Depending on the type of ob the result will be left or rigth
      // justified:
      // Values of most wrapper classes: rigth justified
      // Boolean and Character values  : left  justified
      // Everything else               : left  justified

      // The value null shall be represented by
      // the unquoted String "null"     of length 4, not by
      // the   quoted String "\"null\"" of length 6.
      // Therefore it is replaced by a StringBuilder, not a String:
      if (ob==null) ob = new StringBuilder("null");
      String s = tos(ob);

      Class<?> cl          = ob.getClass();
      boolean  adjustRight = wpTypeList.contains(cl);
      if (cl==Character.class) adjustRight=false;
      if (cl==  Boolean.class) adjustRight=false;
      String rightOrLeft = adjustRight ? "" : "-";

      String fmt = "%" + rightOrLeft + width + "s";
      return String.format(fmt, s);

   } // tos
   // ---------------------------------------------------------------------
   public String repCrAndNl(String s) {
      // Replaces each carriage return character in s by "'\r'" and
      // each newline character by "'\n'". If the result is printed,
      // it will appear as only one line.

      return s.replace("\r", "\\r").replace("\n", "\\n");
   } // repCrAndNl
   // ---------------------------------------------------------------------
   static public String totalsString() {
      // This class contains class fields totalCalls and totalFails.
      // Their values are represented as a readable string and returned.
      return String.format("*** Total calls: %2d, failures: %2d",
         totalCalls, totalFails);
   } // printGrandTotals

   static public int getTotalCalls() {return totalCalls;}
   static public int getTotalFails() {return totalFails;}
   // ---------------------------------------------------------------------
   private void err(String fmt, Object... obA) throws Error {
      // Throws an exception of type Error with a formatted message
      // in it. In this program exceptions of type Error are never
      // caught and always cause the program to halt.
      throw new Error(String.format(fmt, obA));
   } // err
   // =====================================================================
   // The following methods tst01, tst02 and main together are a minimal
   // test of this class (more tests may be found in the files
   // MethodTestTstA.java ...

   static void tst01() {
      // Tests the object function charAt declared in class StringBuilder.

      Object[][] aa = {
         {new StringBuilder("ABC"), "charAt"},
         {'A', 0}, // correct expected result 'A' for param 0
         {'X', 1}, // wrong   expected result 'X' for param 1
         {'C', 2}, // correct expected result 'C' for param 2
         {'Z', 2}, // wrong   expected result 'Z' for param 2
         {'A', 3}, // param  3 is out of range
         {'A',-2}, // param -2 is out of range
      };
      MethodTest mt = new MethodTest(aa);

   } // tst01
   // ---------------------------------------------------------------------
   static void tst03() {
      // Tests a class function max declared in class Math.

      Object[][] aa = {
         {Math.class, "max"},
         {+5, +3, +5}, // correct expected result +5 for params (+3, +5)
         {+5, +5, +3},
         {-4, -3, -5}, // wrong   expected result -4 for params (-3, -5)
         {-3, -5, -3},
         { 0,  1, -5}, // wrong   expected result  0 for params ( 1, -5)
         { 3, -5,  1}, // wrong   expected result  3 for params (-5,  1)
         {1234567890, 1,  1234567890},
      };

      MethodTest mt = new MethodTest(aa);

   } // tst03
   // ---------------------------------------------------------------------
   static public void main(String[] _) {
      printf("MethodTest: Here we go!%n");

      tst01();
      tst03();
      printf("--------------------------------------%n");
      printf("MethodTest: That's all!%n");
   } // main
   // ---------------------------------------------------------------------
   // A method with a short name:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
/*
   // Several methods with schort names:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
*/
   // ---------------------------------------------------------------------
} // class MethodTest
/* ------------------------------------------------------------------------
Output of this program:

MethodTest: Here we go!
--------------------------------------
Class : java.lang.StringBuilder, Method: charAt
Object: "ABC"

"ABC".charAt( 1): xp:'X', is:'B'                             ****
"ABC".charAt( 2): xp:'Z', is:'C'                             ****
"ABC".charAt( 3): xp:'A', is:"String index out of range: 3"  ****
"ABC".charAt(-2): xp:'A', is:"String index out of range: -2" ****

Number of calls:  6, failures:  4
--------------------------------------
Class : java.lang.Math, Method: max

Math.max(-3,-5): xp:-4, is:-3 ****
Math.max( 1,-5): xp: 0, is: 1 ****
Math.max(-5, 1): xp: 3, is: 1 ****

Number of calls:  7, failures:  3
--------------------------------------
MethodTest: That's all!
------------------------------------------------------------------------ */
