// Datei MethodTestTstA.java
/* ------------------------------------------------------------------------
This program demonstrates how to test methods using the class MethodTest.
As a side effect it tests the class MethodTest.
---------------------------------------------------------------------------
To test a method (with the help of the class MethodTest), the necessary
data has to be defined as a variable of type Object[][]. In the example
below this variable is called aa (like "array of arrays").
===========================================================================
Example: Data array aa to test the object method (non-static method)
charAt declared in class String:

Object[][] aa = {
   {"ABC", "charAt"}, // aa[0]: A String object and the method name
   {'A', 0},          // aa[1]: Expected result 'A' for param 0
   {'B', 1},          // aa[2]: Expected result 'B' for param 1
   {'C', 2},          // aa[3]: Expected result 'C' for param 2
};

This array aa describes 3 test calls of the object function charAt declared
in class String. The component aa[0] contains information common to all
calls. The remaining components aa[1] to aa[3] each describe one call of
the function charAt.

Since an object method of class String is to be testet, a String object
has to be specified as component aa[0][0]. The String "ABC" is specified
in this example. It is followed by the name of the method to be tested.

The description of a test call consists of an expected result, followed by
parameters which should "lead to" this result. In the example the component
aa[1] tests, whether the call "ABC".charAt(0) really produces the expected
result 'A'.

In addition to the declaration of the array aa only one more instruction
is needed:

MethodTest mt = new MethodTest(aa);

This instruction will cause the test calls specified in aa to be executed
and the results to be output (to standard output) as follows (5 lines):

--------------------------------------
Class : java.lang.String
Object: "ABC"

Number of calls:  3, failures:  0

By default only failures will cause detailed output, test calls which
produce the expected result will be reported only summarily. The instruction

MethodTest mt = new MethodTest(aa, false);

will output information about every test call (failure or not):

--------------------------------------
Class : java.lang.String
Object: "ABC"

"ABC".charAt(0): xp:'A', is:'A' o.k.
"ABC".charAt(1): xp:'B', is:'B' o.k.
"ABC".charAt(2): xp:'C', is:'C' o.k.

Number of calls:  3, failures:  0

In such an output, "xp" and "is" mean "expected result" and "result as is"
respectively.

The method charAt in class String does not contain any known errors. But we
still can produce failures by specifying wrong values as expected results.
Thus we may produce the following output (11 lines):

--------------------------------------
Class : java.lang.String
Object: "ABC"

"ABC".charAt( 0): xp:'A', is:'A'                             o.k.
"ABC".charAt( 1): xp:'X', is:'B'                             ****
"ABC".charAt( 2): xp:'C', is:'C'                             o.k.
"ABC".charAt( 2): xp:'Z', is:'C'                             ****
"ABC".charAt( 3): xp:'A', is:"String index out of range: 3"  ****
"ABC".charAt(-2): xp:'A', is:"String index out of range: -2" ****

Number of calls:  6, failures:  4
===========================================================================
More examples may be found in the files MethodTestTstB.java, ...
MethodTestD.java

This class MethodTestTstA contains 5 methods tst01, ..., tst05:

tst01 tests the object function  charAt    in class String
tst02 tests an  object function  substring in class String
tst03 tests a   class  function  max       in class Math
tst04 tests the object procedure proc03    in this class MethodTestTstA
tst05 tests the class  procedure proc04    in this class MethodTestTstA

In some of the tests, wrong values have been specified as expected results
in order to cause these tests to fail, although the methods tested do not
contain errors.

Terminology:
+------------------+-------------------------------+
|Terms used here:  |  Translation into C/C++ speak:|
+------------------+-------------------------------+
|method            |                      function |
|function          |  non-void            function |
|procedure         |  void                function |
|object method     |  non-static          function |
|class  method     |  static              function |
|object function   |  non-static non-void function |
|object procedure  |  non-static void     function |
|class  function   |  static non-void     function |
|class  procedure  |  static void         function |
+------------------+-------------------------------+
------------------------------------------------------------------------ */
import        de.bht_berlin.fb6.MethodTest;
import static de.bht_berlin.fb6.MethodTest.totalsString;
import static de.bht_berlin.fb6.MethodTest.p;
import static de.bht_berlin.fb6.MethodTest.pln;

public class MethodTestTstA {
   // ---------------------------------------------------------------------
   static final boolean FAILURES_ONLY = true;
   // ---------------------------------------------------------------------
   static void tst01() {
      // Tests the object function  char charAt(int n)
      // declared in class String.

      Object[][] aa = {
         {new String("ABC"), "charAt"},
         {'A', 0}, // correct expected result 'A' for param 0
         {'X', 1}, // wrong   expected result 'X' for param 1
         {'C', 2}, // correct expected result 'C' for param 2
         {'Z', 2}, // wrong   expected result 'Z' for param 2
         {'A', 3}, // param  3 is out of range
         {'A',-2}, // param -2 is out of range

      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);
   } // tst01
   // ---------------------------------------------------------------------
   static void tst02() {
      // Tests the object method  String substring(int start, int end)
      // declared in class String.

      Object[][] aa = {
         {new String("ABCDEF"), "substring"},
         { "BCD"  , 1, 4},
         {"ABCDEX", 0, 6},
         {"XBC"   , 0, 3},
         {  "CDEF", 2, 6},
         {"X"     , 0, 1},
         {     "F", 5, 6},
         {""      , 2, 2},
      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);
   } // tst02
   // ---------------------------------------------------------------------
   static void tst03() {
      // Tests the class function  int max(int n1, int n2)
      // declared in class Math.

      Object[][] aa = {
         {Math.class, "max"},
         {+5, +3, +5}, // correct expected result +5 for params (+3, +5)
         {+5, +5, +3},
         {-4, -3, -5}, // wrong   expected result -4 for params (-3, -5)
         {-3, -5, -3},
         { 0,  1, -5}, // wrong   expected result  0 for params ( 1, -5)
         { 3, -5,  1}, // wrong   expected result  3 for params (-5,  1)
      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);
   } // tst03
   // ---------------------------------------------------------------------
   static void tst04() {
      // Tests the object procedure proc03 in this class MethodTestTstA.
      // The procedure proc03 outputs its "result" with a method named p
      // to a screen (normally to the real one, but while testing to the
      // internal screen iScreen in Class MethodTest).

      Object[][] aa = {
         {new MethodTestTstA(), "proc03"},
         {"EUR 10", "EUR ", 10}, // correct expected result "EUR 10"
                                 // for params ("EUR ", 10)
         {"USD 15", "USD" , 15}, // wrong   expected result "USD 15"
                                 // for params ("USD" , 15),
                                 // a space is missing after USD
      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);
   } // tst04
   // ---------------------------------------------------------------------
   static void tst05() {
      // Tests the class procedure proc04 in this class MethodTestTstA.
      // The procedure proc04 outputs its "result" to a screen (normally
      // to the real one, but while testing to the internal screen iScreen
      // in Class MethodTest).

      Object[][] aa = {
         {MethodTestTstA.class, "proc04"},
         {"20 EUR\n", " EUR", 20}, // correct expected result "20 EUR"
                                   // for params ("EUR ", 20)
         {"30 USD\n", "USD" , 30}, // wrong   expected result "30 USD"
                                   // for params ("USD" , 30),
                                   // a space is missing before USD
      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);
   } // tst05
   // ---------------------------------------------------------------------
   // In the following methods proc03 and proc04, the names "p" and "pln"
   // denote differnts sets of print methods dpending on circumstances:
   //
   // For production:       For testing:
   // MethodTestTstA.p      MethodTest.p
   // MethodTestTstA.pln    MethodTest.pln

   public void proc03(String s, int n) {
      p(s + n);
   } // proc03
   // ---------------------------------------------------------------------
   static public void proc04(String s, int n) {
      pln(n + s);
   } // proc04
   // ---------------------------------------------------------------------
   static public void main(String[] _) {
      printf("MethodTestTstA: Here we go!%n");

      tst01();
      tst02();
      tst03();

      tst04();
      tst05();

      printf("====================================%n");
      printf("%s%n", totalsString());
      printf("====================================%n");
      printf("MethodTestTstA: That's all!%n");
   } // main
   // ---------------------------------------------------------------------
   public String toString() {return "The best of its class!";}
   // ---------------------------------------------------------------------
   // A method with a short name:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
/*
   // Several methods with short names, printing to the real screen.
   // Should be commented out during testing and activated for production:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
*/
      // ---------------------------------------------------------------------
} // class MethodTestTstA
/* ------------------------------------------------------------------------
Output of this program (with FAILURES_ONLY declared to be false):

MethodTestTstA: Here we go!
--------------------------------------
Class : java.lang.String, Method: charAt
Object: "ABC"

"ABC".charAt( 0): xp:'A', is:'A'                             o.k.
"ABC".charAt( 1): xp:'X', is:'B'                             ****
"ABC".charAt( 2): xp:'C', is:'C'                             o.k.
"ABC".charAt( 2): xp:'Z', is:'C'                             ****
"ABC".charAt( 3): xp:'A', is:"String index out of range: 3"  ****
"ABC".charAt(-2): xp:'A', is:"String index out of range: -2" ****

Number of calls:  6, failures:  4
--------------------------------------
Class : java.lang.String, Method: substring
Object: "ABCDEF"

"ABCDEF".substring(1,4): xp:"BCD"   , is:"BCD"    o.k.
"ABCDEF".substring(0,6): xp:"ABCDEF", is:"ABCDEF" o.k.
"ABCDEF".substring(0,3): xp:"ABC"   , is:"ABC"    o.k.
"ABCDEF".substring(2,6): xp:"CDEF"  , is:"CDEF"   o.k.
"ABCDEF".substring(0,1): xp:"A"     , is:"A"      o.k.
"ABCDEF".substring(5,6): xp:"F"     , is:"F"      o.k.
"ABCDEF".substring(2,2): xp:""      , is:""       o.k.

Number of calls:  7, failures:  0
--------------------------------------
Class : java.lang.Math, Method: max

Math.max( 3, 5): xp: 5, is: 5 o.k.
Math.max( 5, 3): xp: 5, is: 5 o.k.
Math.max(-3,-5): xp:-4, is:-3 ****
Math.max(-5,-3): xp:-3, is:-3 o.k.
Math.max( 1,-5): xp: 0, is: 1 ****
Math.max(-5, 1): xp: 3, is: 1 ****

Number of calls:  6, failures:  3
--------------------------------------
Class : MethodTestTstA, Method: proc03
Object: [The best of its class!]

[The best of its class!].proc03("EUR ",10): xp:"EUR 10", is:"EUR 10" o.k.
[The best of its class!].proc03("USD" ,15): xp:"USD 15", is:"USD15"  ****

Number of calls:  2, failures:  1
--------------------------------------
Class : MethodTestTstA, Method: proc04

MethodTestTstA.proc04(" EUR",20): xp:"20 EUR", is:"20 EUR" o.k.
MethodTestTstA.proc04("USD" ,30): xp:"30 USD", is:"30USD"  ****

Number of calls:  2, failures:  1
====================================
*** Total calls: 23, failures:  9
====================================
MethodTestTstA: That's all!
---------------------------------------------------------------------------
Output of this program (with FAILURES_ONLY declared to be true):

MethodTestTstA: Here we go!
--------------------------------------
Class : java.lang.String, Method: charAt
Object: "ABC"

"ABC".charAt( 1): xp:'X', is:'B'                             ****
"ABC".charAt( 2): xp:'Z', is:'C'                             ****
"ABC".charAt( 3): xp:'A', is:"String index out of range: 3"  ****
"ABC".charAt(-2): xp:'A', is:"String index out of range: -2" ****

Number of calls:  6, failures:  4
--------------------------------------
Class : java.lang.String, Method: substring
Object: "ABCDEF"

Number of calls:  7, failures:  0
--------------------------------------
Class : java.lang.Math, Method: max

Math.max(-3,-5): xp:-4, is:-3 ****
Math.max( 1,-5): xp: 0, is: 1 ****
Math.max(-5, 1): xp: 3, is: 1 ****

Number of calls:  6, failures:  3
--------------------------------------
Class : MethodTestTstA, Method: proc03
Object: [The best of its class!]

[The best of its class!].proc03("USD",15): xp:"USD 15", is:"USD15" ****

Number of calls:  2, failures:  1
--------------------------------------
Class : MethodTestTstA, Method: proc04

MethodTestTstA.proc04("USD",30): xp:"30 USD", is:"30USD" ****

Number of calls:  2, failures:  1
====================================
*** Total calls: 23, failures:  9
====================================
MethodTestTstA: That's all!
------------------------------------------------------------------------ */