// Datei MethodTestTstB.java
/* ------------------------------------------------------------------------
This program demonstrates how to test methods using the class MethodTest.
As a side effect it tests the class MethodTest.
---------------------------------------------------------------------------
To test a method (with the help of the class MethodTest), the necessary
data has to be defined as a variable of type Object[][]. In each of the
examples below this variable is called aa (like "array of arrays").
===========================================================================
Example 1: Data array aa to test the object method (non-static method)
charAt declared in class String:

Object[][] aa = {
   {"ABC", "charAt"}, // A String object and the method name
   {'A', 0},          // Expected result 'A' for param 0
   {'B', 1},          // Expected result 'B' for param 1
   {'C', 2},          // Expected result 'C' for param 2
};

This array aa describes 3 test calls of the object function charAt declared
in class String. The component aa[0] contains information common to all
calls. The remaining components aa[1] to aa[3] describe one call each.

Since an object method of class String is to be testet, a String object
has to be specified as component aa[0][0]. The String "ABC" is specified
in this example. It is followed by the name of the method to be tested as
component aa[0][1].

The description of a test call consists of an expected result, followed by
parameters which should "lead to" this result. In the example the component
aa[1] tests, whether the call "ABC".charAt(0) really produces the expected
result 'A'.

In addition to the declaration of the array aa only one more instruction
is needed:

MethodTest mt = new MethodTest(aa);

This instruction will cause the test calls specified in aa to be executed
and the results to be output (to standard output) as follows (5 lines):

--------------------------------------
Class : java.lang.String
Object: "ABC"

Number of calls:  3, failures:  0

By default only failures will cause detailed output, test calls which
produce the expected result will be reported only summarily. The instruction

MethodTest mt = new MethodTest(aa, false);

will output information about each test call (failure or not):

--------------------------------------
Class : java.lang.String
Object: "ABC"

"ABC".charAt(0): xp:'A', is:'A' o.k.
"ABC".charAt(1): xp:'B', is:'B' o.k.
"ABC".charAt(2): xp:'C', is:'C' o.k.

Number of calls:  3, failures:  0

In such an output, "xp" ans "is" mean "expected result" and "result as is"
respectively.

The method charAt in class String does not contain any known errors. But we
still can produce failures by specifying wrong values as expected results.
Thus we may produce the following output (11 lines):

--------------------------------------
Class : java.lang.String
Object: "ABC"

"ABC".charAt( 0): xp:'A', is:'A'                             o.k.
"ABC".charAt( 1): xp:'X', is:'B'                             ****
"ABC".charAt( 2): xp:'C', is:'C'                             o.k.
"ABC".charAt( 2): xp:'Z', is:'C'                             ****
"ABC".charAt( 3): xp:'A', is:"String index out of range: 3"  ****
"ABC".charAt(-2): xp:'A', is:"String index out of range: -2" ****

Number of calls:  6, failures:  4

===========================================================================
Example 2: Specification and execution of 6 test calls of a
class function (static function) named max declared in class Math:

Object[][] aa = {
   {Math.class, "max"},  // The class object of Math and method name
   {+5, +3, +5},         // Test call 1
   {+5, +5, +3},         // Test call 2
   {-4, -3, -5},         // Test call 3
   {-3, -5, -3},         // Test call 4
   { 0,  1, -5},         // Test call 5
   { 3, -5,  1},         // Test call 6
};

MethodTest mt = new MethodTest(aa);


The method max has two parameters and returns the greater of the two.
More precisely: For each numeric (primitive) type there is a max
method. In the above example






To execute the tests specified by the array aa and to output the results
only one more instruction is needed:






The component aa[0] describes the method to be tested.
Each of the remaining components aa[1], aa[2], ... describes one
call of this method (the expected result and the parameters).

Object methods (non-static methods) are described differently from
class  methods (    static methods), as follows:

Minimal description of an OBJECT METHOD to be tested:
aa[0][0]: An object, which contains the method to be tested.
aa[0][1]: The name of the method to be tested (as a String).

Minimal description of a CLASS METHOD to be tested:
aa[0][0]: A Class object, which contains the method to be tested.
aa[0][1]: The name of the method to be tested (as a String)

If such a minimal description is not enough to characterize a
unique method, the parameter types of the method have to be
specified by additional components of aa[0]:
aa[0][2]: Type of first  parameter (as a Class-object)
aa[0][3]: Type of second parameter (as a Class-object)
...

A call of the method to be tested has to be described by a component
aa[i] (with i greater than 0) as follows:
aa[i][0]: The expected result  of the call
aa[i][1]: The first  parameter of the call
aa[i][2]: The second parameter of the call
...

The class MethodTest contains two nested classes (as static members) named
Calls and Call. A Calls object is a kind of container for Call objects.

A Calls object may (only) be created with one of the factory methods
makeStaticCalls and makeNonStaticCalls. Each of these methods takes a
single parameter of type Object[][] and this parameter should contain
data as described above for the array aa. The parameter for makeStatic-
Calls should describe tests of a static method (class method). The para-
meter for makeNonStaticCalls should describe tests of a non-static method
(object method).

Each call of the factory methods (makeStaticCalls or makeNonStaticCalls)
not only creates a Calls object with Call objects inside, but
automatically executes each Call object and stores the result and a
verdict (isOk or isFailure) inside each Call object.

The class MethodTest provides 4 methods for viewing a Calls object
(i.e. the results and verdicts of all the Call objects it contains):
printAll, printFailures, printAllCompact and printFailuresCompact.
The compact versions print 1 call per line,
the non-compact versions use 3 lines per call.

This class MethodTestTstB contains 4 methods tst01, ..., tst04:

tst01 tests the object function  charAt in class StringBuilder (1 int param)
tst02 tests the class  function  max    in class Math          (2 int param)
tst03 tests the object procedure proc03 in this class MethodTestTstB
tst04 tests the class  procedure proc04 in this class MethodTestTstB

In some of the tests, wrong values have been specified as expected results
in order to cause these tests to fail, although the methods tested do not
contain errors.
------------------------------------------------------------------------ */
import        de.bht_berlin.fb6.MethodTest;
import static de.bht_berlin.fb6.MethodTest.totalsString;
import static de.bht_berlin.fb6.MethodTest.p;
import static de.bht_berlin.fb6.MethodTest.pln;

public class MethodTestTstB {
   // ---------------------------------------------------------------------
   static final boolean FAILURES_ONLY = false;
   // ---------------------------------------------------------------------
   static void tst01() {
      // Tests the object function  charAt in class StringBuilder.
      // The name "charAt" is enough to specify this method uniquely.

      Object[][] aa = {
         {new String("ABC"), "charAt"},
         {'A', 0}, // correct expected result 'A' for param 0
         {'X', 1}, // wrong   expected result 'X' for param 1
         {'C', 2}, // correct expected result 'C' for param 2
         {'Z', 2}, // wrong   expected result 'Z' for param 2
         {'A', 3}, // param  3 is out of range
         {'A',-2}, // param -2 is out of range

      };

      MethodTest mt = new MethodTest(aa, false);

   } // tst01
   // ---------------------------------------------------------------------

   static void tst02() {
      // Tests an object method named substring in class String.
      //
      // Althoug there are two methods named substring in class String
      // it is enough to specify its name "substring" here, since the
      // two methods have different numbers of parameters.

      Object[][] aa = {
         {new String("ABCDEF"), "substring"},
         { "BCD"  , 1, 4},
         {"ABCDEF", 0, 6},
         {"ABC"   , 0, 3},
         {  "CDEF", 2, 6},
         {"A"     , 0, 1},
         {     "F", 5, 6},
         {""      , 2, 2},
      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);

   } // tst02
   // ---------------------------------------------------------------------
   static void tst03() {
      // Tests a class function max in class Math.
      //
      // Since in class Math there are several methods named max with two
      // parameters, the method to be tested is described here by its name
      // "max" and the types of its two parameters:

      Object[][] aa = {
         {Math.class, "max"},
         {+5.0, +3.0, +5.0}, // correct expected result +5 for params (+3, +5)
         {+5.0, +5, +3},
         {-4.0, -3, -5}, // wrong   expected result -4 for params (-3, -5)
         {-3.0, -5, -3},
         { 0.0,  1, -5}, // wrong   expected result  0 for params ( 1, -5)
         { 3.0, -5,  1}, // wrong   expected result  3 for params (-5,  1)
      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);

   } // tst03
   // ---------------------------------------------------------------------
   static void tst04() {
      // Tests the object procedure proc03 in this class MethodTestTstB.
      // The procedure proc03 outputs its "result" to a screen (normally
      // to the real one, but while testing to the internal screen iScreen
      // in Class MethodTest).

      Object[][] aa = {
         {new MethodTestTstB(), "proc03"},
         {"EUR 10", "EUR ", 10}, // correct expected result "EUR 10"
                                 // for params ("EUR ", 10)
         {"USD 15", "USD" , 15}, // wrong   expected result "USD 15"
                                 // for params ("USD" , 15),
                                 // a space is missing after USD
      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);

   } // tst04
   // ---------------------------------------------------------------------
   static void tst05() {
      // Tests the class procedure proc04 in this class MethodTestTstB.
      // The procedure proc04 outputs its "result" to a screen (normally
      // to the real one, but while testing to the internal screen iScreen
      // in Class MethodTest).

      Object[][] aa = {
         {MethodTestTstB.class, "proc04"},
         {"20 EUR", " EUR", 20}, // correct expected result "20 EUR"
                                 // for params ("EUR ", 20)
         {"30 USD", "USD" , 30}, // wrong   expected result "30 USD"
                                 // for params ("USD" , 30),
                                 // a space is missing before USD
      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);

   } // tst05
   // ---------------------------------------------------------------------
   static void tst06() {

      Object[][] aa = {
         {"0123456789", "indexOf"},
         { -1,        "ABC"},
         {  0, "0123456789"},
         {  0, "01"        },
         {  5,      "567"  },
      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);

   } // tst06
   // ---------------------------------------------------------------------
   static void tst07() {

      Object[][] aa = {
         {"0123456789", "indexOf", String.class},
         { -1,        "ABC"},
         {  0, "0123456789"},
         {  0, "01"        },
         {  5,      "567"  },
      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);

   } // tst07
   // ---------------------------------------------------------------------
   static void tst08() {

      Object[][] aa = {
         {"0123456789", "charAt", Integer.TYPE},
         { '0',  0},
         { '0',  1},
         { '1',  0},
         { '1', 10},
      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);

   } // tst08
   // ---------------------------------------------------------------------
   static void tst09() {

      Object[][] aa = {
         {new MethodTestTstB(), "plus", Integer.class, Integer.class},
         {  2,   1,  1},
      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);

   } // tst09
   // ---------------------------------------------------------------------
   static void tst10() {

      Object[][] aa = {
         {Number.class, "isInstance", Object.class},
         {  true,   123},
         {  false, true},
         {  true,   4.5},
         {  false,  4.5}, // Wrong expected result fals, failure!
      };

      MethodTest mt = new MethodTest(aa);

   } // tst10
   // ---------------------------------------------------------------------
   static void tst11() {
      // Tests the class function proc05 in this class MethodTestTstB.

      Object[][] aa = {
         {new MethodTestTstB(), "proc05"},
         {new int[]{5, 10, 15}, new int[]{10, 20, 30}},
         {new int[]{5}        , new int[]{10}        },
         {new int[]{}         , new int[]{}          },
         {null                , null                 },
      };

      MethodTest mt = new MethodTest(aa, FAILURES_ONLY);
   } // tst11
   // ---------------------------------------------------------------------
   // In the following methods proc03 and proc04 the name "p"
   // denotes a print method, either
   // MethodTestTstB.p     (for production) or
   // MethodTest.p        (for testing).

   public void proc03(String s, int n) {
      p(s + n);
   } // proc03
   // ---------------------------------------------------------------------
   static public void proc04(String s, int n) {
      p(n + s);
   } // proc04
   // ---------------------------------------------------------------------
   static public int[] proc05(int[] ir) {
      if (ir == null) return null;
      int[] res = new int[ir.length];
      for (int i=0; i<ir.length; i++) res[i] = ir[i] / 2;
      return res;
   } // proc05
   // ---------------------------------------------------------------------
   // A function with parameters and return value of a wrapper class:
   public Integer plus(Integer n1, Integer n2) {
      return n1 + n2;
   } // plus
   // ---------------------------------------------------------------------
   static public void main(String[] _) {
      printf("MethodTestTstB: Here we go!%n");

      tst01();
      tst02();

      tst03();

      tst04();
      tst05();
      tst06();
      tst07();
      tst08();

      tst09();

      tst10();
      tst11();


      printf("====================================%n");
      printf("%s%n", totalsString());
      printf("====================================%n");
      printf("MethodTestTstB: That's all!%n");
   } // main
   // ---------------------------------------------------------------------
   public String toString() {return "The best of its class!";}
   // ---------------------------------------------------------------------
   // A method with a short name:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
/*
   // Several methods with short names, printing to the real screen.
   // Should be commented out during testing and activated for production:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
*/
      // ---------------------------------------------------------------------
} // class MethodTestTstB
/* ------------------------------------------------------------------------
Output of this program:

MethodTestTstB: Here we go!
--------------------------------------
Class : java.lang.String, Method: charAt
Object: "ABC"

"ABC".charAt( 0): xp:'A', is:'A'                             o.k.
"ABC".charAt( 1): xp:'X', is:'B'                             ****
"ABC".charAt( 2): xp:'C', is:'C'                             o.k.
"ABC".charAt( 2): xp:'Z', is:'C'                             ****
"ABC".charAt( 3): xp:'A', is:"String index out of range: 3"  ****
"ABC".charAt(-2): xp:'A', is:"String index out of range: -2" ****

Number of calls:  6, failures:  4
--------------------------------------
Class : java.lang.String, Method: substring
Object: "ABCDEF"

"ABCDEF".substring(1,4): xp:"BCD"   , is:"BCD"    o.k.
"ABCDEF".substring(0,6): xp:"ABCDEF", is:"ABCDEF" o.k.
"ABCDEF".substring(0,3): xp:"ABC"   , is:"ABC"    o.k.
"ABCDEF".substring(2,6): xp:"CDEF"  , is:"CDEF"   o.k.
"ABCDEF".substring(0,1): xp:"A"     , is:"A"      o.k.
"ABCDEF".substring(5,6): xp:"F"     , is:"F"      o.k.
"ABCDEF".substring(2,2): xp:""      , is:""       o.k.

Number of calls:  7, failures:  0
--------------------------------------
Class : java.lang.Math, Method: max

Math.max(3.0,5.0): xp: 5.0, is: 5.0 o.k.
Math.max(  5,  3): xp: 5.0, is: 5.0 o.k.
Math.max( -3, -5): xp:-4.0, is:-3.0 ****
Math.max( -5, -3): xp:-3.0, is:-3.0 o.k.
Math.max(  1, -5): xp: 0.0, is: 1.0 ****
Math.max( -5,  1): xp: 3.0, is: 1.0 ****

Number of calls:  6, failures:  3
--------------------------------------
Class : MethodTestTstB, Method: proc03
Object: The best of its class!

The best of its class!.proc03("EUR ",10): xp:"EUR 10", is:"EUR 10" o.k.
The best of its class!.proc03("USD" ,15): xp:"USD 15", is:"USD15"  ****

Number of calls:  2, failures:  1
--------------------------------------
Class : MethodTestTstB, Method: proc04

MethodTestTstB.proc04(" EUR",20): xp:"20 EUR", is:"20 EUR" o.k.
MethodTestTstB.proc04("USD" ,30): xp:"30 USD", is:"30USD"  ****

Number of calls:  2, failures:  1
--------------------------------------
Class : java.lang.String, Method: indexOf
Object: "0123456789"

"0123456789".indexOf("ABC"       ): xp:-1, is:-1 o.k.
"0123456789".indexOf("0123456789"): xp: 0, is: 0 o.k.
"0123456789".indexOf("01"        ): xp: 0, is: 0 o.k.
"0123456789".indexOf("567"       ): xp: 5, is: 5 o.k.

Number of calls:  4, failures:  0
--------------------------------------
Class : java.lang.String, Method: indexOf
Object: "0123456789"

"0123456789".indexOf("ABC"       ): xp:-1, is:-1 o.k.
"0123456789".indexOf("0123456789"): xp: 0, is: 0 o.k.
"0123456789".indexOf("01"        ): xp: 0, is: 0 o.k.
"0123456789".indexOf("567"       ): xp: 5, is: 5 o.k.

Number of calls:  4, failures:  0
--------------------------------------
Class : java.lang.String, Method: charAt
Object: "0123456789"

"0123456789".charAt( 0): xp:'0', is:'0'                             o.k.
"0123456789".charAt( 1): xp:'0', is:'1'                             ****
"0123456789".charAt( 0): xp:'1', is:'0'                             ****
"0123456789".charAt(10): xp:'1', is:"String index out of range: 10" ****

Number of calls:  4, failures:  3
--------------------------------------
Class : MethodTestTstB, Method: plus
Object: The best of its class!

The best of its class!.plus(1,1): xp:2, is:2 o.k.

Number of calls:  1, failures:  0
--------------------------------------
Class : java.lang.Number, Method: isInstance
Object: class java.lang.Number

class java.lang.Number.isInstance(4.5): xp:false, is:true ****

Number of calls:  4, failures:  1
--------------------------------------
Class : MethodTestTstB, Method: proc05

MethodTestTstB.proc05([10, 20, 30]): xp:[5, 10, 15], is:[5, 10, 15] o.k.
MethodTestTstB.proc05([10]        ): xp:[5]        , is:[5]         o.k.
MethodTestTstB.proc05([]          ): xp:[]         , is:[]          o.k.
MethodTestTstB.proc05(null        ): xp:null       , is:null        o.k.

Number of calls:  4, failures:  0
====================================
*** Total calls: 44, failures: 13
====================================
MethodTestTstB: That's all!
------------------------------------------------------------------------ */
