// Datei ToString.java
/* ------------------------------------------------------------------------
Diese Klasse enthaelt zwei Klassenmethoden (ToString.sts und ToString.uts)
mit denen man aus beliebigen Objekten eine String-Darstellung erzeugen kann.
Falls das betreffende Objekt eine Reihung ist, bekommt man eine "brauchbare"
String-Darstellung (nicht so etwas wie "[I@19b49e6" sondern "[2, 3, 1]").
Die Methode sts erzeugt eine Kopie der Reihung, versucht diese zu sortieren
und liefert eine String String-Darstellung der Kopie (z.B. "[1, 2, 3]").
---------------------------------------------------------------------------
Motivation: Die Methoden sts (sorted to string) und uts (unsorted to string)
koennen das Schreiben von JUnit-Tests fuer Methoden mit Reihungen als
Parameter und/oder als Ergebnis erleichtern.
------------------------------------------------------------------------ */
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;
import java.math.BigInteger;

public class ToString {
   // ---------------------------------------------------------------------
   static public String sts(Object ob) {
      // Liefert eine String-Darstellung von ob. Falls ob eine Reihung ist
      // wird versucht, vor dem Erzeugen der String-Darstellung eine Kopie
      // von ob zu sortieren.
      // "sts" soll an "sorted to string" erinnern
      return ts(ob, true);
   } // sts
   // ---------------------------------------------------------------------
   static public String uts(Object ob) {
      // Liefert eine String-Darstellung von ob. Auch wenn ob eine Reihung
      // ist, wird kein Sortier-Versuch gemacht.
      // "uts" soll an "unsorted to string" erinnern.
      return ts(ob, false);
   } // uts
   // ---------------------------------------------------------------------
   static private String ts(Object ob, boolean sort) {
      // Liefert eine String-Darstellung von ob.
      // Wenn ob eine Reihung und sort gleich true ist, wird versucht,
      // eine Kopie von ob zu sortieren, ehe die String-Darstellung
      // der Kopie erzeugt wird.
      // Wenn ob eine Reihung und sort gleich false ist, wird nur
      // eine String-Darstellung von ob erzeugt (ohne Sortierung).
      // Wenn ob keine Reihung ist, wird ob.toString() als Ergebnis
      // geliefert.
      // In keinem Fall wird ob veraendert.
      // ts soll an "to string" erinnern.

      try {
         Class<?> k = ob.getClass();

         // Wenn ob keine Reihung ist:
         if (!k.isArray()) return ob.toString();

         // Wenn ob eine Reihung ist:
         if (k.getComponentType().isPrimitive()) {
            // Wenn die Reihung ob primitive Komponenten hat:
            for (int i=0; i<ptR.length; i++) {
               if (k == ptR[i]) {
                  // Wenn die Reihung ob Komponenten des
                  // primitiven Typs ptR[i] hat:
                  return (String) metR[i].invoke(null, ob, sort);
               }
            }
         }

         // Wenn die Reihung ob nicht-primitve Komponenten hat:
         Object[] obr = (Object[]) ob;
         if (sort) {
            try {
               // Eine Kopie der Reihung obr (alias ob) erzeugen:
               obr = Arrays.copyOf(obr, obr.length);
               // Versuch, obr zu sortieren.
               // Das scheitert, wenn nicht alle Komponenten von obr
               // wechselseitig mit compareTo vergleichbar sind:
               Arrays.sort(obr);
            } catch (ClassCastException ex) {
               ;
            }
         }

         return Arrays.deepToString((Object[]) obr);

      } catch (Exception ex) {
         printf("ex: %s%n", ex);
         return "In Methode sts  trat eine Ausnahme auf!";
      }
   } // ts
   // ---------------------------------------------------------------------
   // Fuer jeden primitiven Typ pt eine Methode, die aus einer Reihung vom
   // Typ pt[] eine (sortierte oder unsortierte) String-Darstellung erzeugt:
   // "suts" soll an "sorted or unsorted to string" erinnern.
   // ---------------------------------------------------------------------
   static private String suts(byte[] r, boolean sort) {
      if (sort) {
         r = Arrays.copyOf(r, r.length);
         Arrays.sort(r);
      }
      return Arrays.toString(r);
   } // suts

   static private String suts(char[] r, boolean sort) {
      if (sort) {
         r = Arrays.copyOf(r, r.length);
         Arrays.sort(r);
      }
      return Arrays.toString(r);
   } // suts

   static private String suts(short[] r, boolean sort) {
      if (sort) {
         r = Arrays.copyOf(r, r.length);
         Arrays.sort(r);
      }
      return Arrays.toString(r);
   } // suts

   static private String suts(int[] r, boolean sort) {
      if (sort) {
         r = Arrays.copyOf(r, r.length);
         Arrays.sort(r);
      }
      return Arrays.toString(r);
   } // suts

   static private String suts(long[] r, boolean sort) {
      if (sort) {
         r = Arrays.copyOf(r, r.length);
         Arrays.sort(r);
      }
      return Arrays.toString(r);
   } // suts

   static private String suts(float[] r, boolean sort) {
      if (sort) {
         r = Arrays.copyOf(r, r.length);
         Arrays.sort(r);
      }
      return Arrays.toString(r);
   } // suts

   static private String suts(double[] r, boolean sort) {
      if (sort) {
         r = Arrays.copyOf(r, r.length);
         Arrays.sort(r);
      }
      return Arrays.toString(r);
   } // suts

   static private String suts(boolean[] r, boolean sort) {
      // Fuer Reihungen mit boolean-Komponenten gibt es in der
      // Klasse Arrays keine sort-Methode.
      return Arrays.toString(r);
   } // suts
   // ---------------------------------------------------------------------
   // Eine Reihung mit den Class-Objekten aller 8 primitiven Typen:
   static Class<?>[] ptR = {
      byte   [].class,
      char   [].class,
      short  [].class,
      int    [].class,
      long   [].class,
      float  [].class,
      double [].class,
      boolean[].class,
    };

   // Eine Reihung mit je einem Method-Objekt fuer jeden primitiven Typ:
   static Method[] metR = new Method[ptR.length];

   static {
      // Die Reihung metR wird mit diesem statischen Initialisierer
      // intitialisiert, weil dabei (theoretisch) eine Ausnahme geworfen
      // werden koennte (praktisch kann das kaum passieren).
      try {
         Class<ToString> kts = ToString.class;
         for (int i=0; i<metR.length; i++) {
            metR[i] = kts.getDeclaredMethod("suts", ptR[i], Boolean.TYPE);
         }
      } catch (NoSuchMethodException ex) {
         throw new Error(ex);
      }
   } // statischer Initialisierer
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
   static public void main(String[] _) {
      // Kleiner Test: Die Methoden sts und uts werden auf verschiedene
      // Parameter angewendet und die Ergebnisse werden ausgegeben.

      printf("ToString: Jetzt geht es los!%n");

      // Ein paar Objekte, die keine Reihungen sind:
      String          stri  = "Hallo!";
      StringBuilder   sbld  = new StringBuilder("Hallo!");
      BigInteger      bigi  = new BigInteger("-12345");

      // Ein paar einstufige Reihungen:
      byte         [] byteR = {2, 3, 1};
      char         [] charR = {'C', 'A', 'B'};
      short        [] shrtR = {2, 3, 1};
      int          [] int_R = {2, 3, 1};
      long         [] longR = {2, 3, 1};
      float        [] flotR = {2.0F, 3.0F, 1.0F};
      double       [] doblR = {2.0D, 3.0D, 1.0D};
      boolean      [] boolR = {true, false, true, true};

      // Anmerk: String-Objekte sind NATUERLICH GEORDNET (d.h. die
      // Klasse String implementiert die Schnittstelle Comparable<String>).
      // Dagegen sind StringBuilder-Objekte sind NICHT natuerlich geordnet
      String       [] striR = {"C", "A", "B"};
      StringBuilder[] sbldR = {
         new StringBuilder("C"),
         new StringBuilder("A"),
         new StringBuilder("B"),
      };

      // Ein paar mehrstufige (genauer: zweistufige) Reihungen:
      byte         [][] byteRR = {{2, 3, 1}, {5, 4}, {8}};
      String       [][] striRR = {{"C", "A", "B"}, {"E", "D"}, {"F"}};
      StringBuilder[][] sbldRR = {
         {new StringBuilder("C"),
          new StringBuilder("A"),
          new StringBuilder("A"),
         },
         {new StringBuilder("E"),
          new StringBuilder("D"),
         },
         {new StringBuilder("F"),
         }
      };

      printf("----------------------------------------------------------%n");
      printf("Die Methode sts wird auf nicht-Reihungen angewendet:      %n");
      printf("stri:   %s%n", sts(stri));
      printf("sbld:   %s%n", sts(sbld));
      printf("bigi:   %s%n", sts(bigi));
      printf("----------------------------------------------------------%n");
      printf("Die Methode uts wird auf nicht-Reihungen angewendet:      %n");
      printf("stri:   %s%n", uts(stri));
      printf("sbld:   %s%n", uts(sbld));
      printf("bigi:   %s%n", uts(bigi));
      printf("----------------------------------------------------------%n");
      printf("Die Methode sts wird auf einstufige Reihungen angewendet: %n");
      printf("byteR:  %s%n", sts(byteR));
      printf("charR:  %s%n", sts(charR));
      printf("shrtR:  %s%n", sts(shrtR));
      printf("int_R:  %s%n", sts(int_R));
      printf("longR:  %s%n", sts(longR));
      printf("flotR:  %s%n", sts(flotR));
      printf("doblR:  %s%n", sts(doblR));
      printf("boolR:  %s%n", sts(boolR));
      printf("striR:  %s%n", sts(striR));
      printf("sbldR:  %s%n", sts(sbldR));
      printf("----------------------------------------------------------%n");
      printf("Die Methode uts wird auf einstufige Reihungen angewendet: %n");
      printf("byteR:  %s%n", uts(byteR));
      printf("charR:  %s%n", uts(charR));
      printf("shrtR:  %s%n", uts(shrtR));
      printf("int_R:  %s%n", uts(int_R));
      printf("longR:  %s%n", uts(longR));
      printf("flotR:  %s%n", uts(flotR));
      printf("doblR:  %s%n", uts(doblR));
      printf("boolR:  %s%n", uts(boolR));
      printf("striR:  %s%n", uts(striR));
      printf("sbldR:  %s%n", uts(sbldR));
      printf("----------------------------------------------------------%n");
      printf("Die Methode sts wird auf mehrstufige Reihungen angewendet:%n");
      printf("byteRR: %s%n", sts(byteRR));
      printf("striRR: %s%n", sts(striRR));
      printf("sbldRR: %s%n", sts(sbldRR));
      printf("----------------------------------------------------------%n");
      printf("Die Methode uts wird auf mehrstufige Reihungen angewendet:%n");
      printf("byteRR: %s%n", uts(byteRR));
      printf("striRR: %s%n", uts(striRR));
      printf("sbldRR: %s%n", uts(sbldRR));
      printf("----------------------------------------------------------%n");
      printf("ToString: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
} // class ToString
/* ------------------------------------------------------------------------
Ausgabe des Programms:

ToString: Jetzt geht es los!
----------------------------------------------------------
Die Methode sts wird auf nicht-Reihungen angewendet:
stri:   Hallo!
sbld:   Hallo!
bigi:   -12345
----------------------------------------------------------
Die Methode uts wird auf nicht-Reihungen angewendet:
stri:   Hallo!
sbld:   Hallo!
bigi:   -12345
----------------------------------------------------------
Die Methode sts wird auf einstufige Reihungen angewendet:
byteR:  [1, 2, 3]
charR:  [A, B, C]
shrtR:  [1, 2, 3]
int_R:  [1, 2, 3]
longR:  [1, 2, 3]
flotR:  [1.0, 2.0, 3.0]
doblR:  [1.0, 2.0, 3.0]
boolR:  [true, false, true, true]
striR:  [A, B, C]
sbldR:  [C, A, B]
----------------------------------------------------------
Die Methode uts wird auf einstufige Reihungen angewendet:
byteR:  [2, 3, 1]
charR:  [C, A, B]
shrtR:  [2, 3, 1]
int_R:  [2, 3, 1]
longR:  [2, 3, 1]
flotR:  [2.0, 3.0, 1.0]
doblR:  [2.0, 3.0, 1.0]
boolR:  [true, false, true, true]
striR:  [C, A, B]
sbldR:  [C, A, B]
----------------------------------------------------------
Die Methode sts wird auf mehrstufige Reihungen angewendet:
byteRR: [[2, 3, 1], [5, 4], [8]]
striRR: [[C, A, B], [E, D], [F]]
sbldRR: [[C, A, A], [E, D], [F]]
----------------------------------------------------------
Die Methode uts wird auf mehrstufige Reihungen angewendet:
byteRR: [[2, 3, 1], [5, 4], [8]]
striRR: [[C, A, B], [E, D], [F]]
sbldRR: [[C, A, A], [E, D], [F]]
----------------------------------------------------------
------------------------------------------------------------------------ */
